import { useReducer, useRef, useEffect, createElement } from 'react';
import { Animated } from 'r-animated';

/*! *****************************************************************************
Copyright (c) Microsoft Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

function Carousel(props) {
    var _a = useReducer(function (state, action) {
        switch (action.type) {
            case "NEXT":
                return {
                    active: state.nextActive,
                    nextActive: (state.nextActive + 1) % props.slides.length
                };
            case "BRING_NEXT":
                return {
                    active: state.active,
                    nextActive: state.nextActive
                };
            case "CUSTOM":
                return {
                    active: state.nextActive,
                    nextActive: action.index
                };
            default:
                throw new Error();
        }
    }, { active: -1, nextActive: 0 }), state = _a[0], dispatch = _a[1];
    var timerId1 = useRef();
    var timerId2 = useRef();
    useEffect(function () {
        clearTimeout(timerId1.current);
        timerId1.current = setTimeout(function () {
            dispatch({ type: "NEXT" });
        }, props.duration || 1000);
        return function () { return clearTimeout(timerId1.current); };
    }, [state.nextActive, props.duration]);
    useEffect(function () {
        clearTimeout(timerId2.current);
        timerId2.current = setTimeout(function () {
            dispatch({ type: "BRING_NEXT" });
        }, props.duration || 1000);
        return function () { return clearTimeout(timerId2.current); };
    }, [state.active, props.duration]);
    function setIndex(index) {
        clearTimeout(timerId1.current);
        clearTimeout(timerId2.current);
        dispatch({ type: "CUSTOM", index: index });
        timerId1.current = setTimeout(function () {
            dispatch({ type: "BRING_NEXT" });
        }, props.duration || 1000);
    }
    return (createElement("div", { style: styles.container, className: "animated-carousel-container", key: state.nextActive + 10 },
        createElement(Animated, __assign({ duration: props.animationDuration, timingFunction: props.animationTimingFunction }, animationTypes[props.animationType] &&
            animationTypes[props.animationType].active, props.customAnimation && props.customAnimation.active, { delay: props.animationDelay || 100 }), function (style) { return (createElement("div", { style: __assign({}, styles.item, style), className: "animated-carousel-item" }, props.slides[state.active])); }),
        createElement(Animated, __assign({ duration: props.animationDuration, timingFunction: props.animationTimingFunction }, animationTypes[props.animationType] &&
            animationTypes[props.animationType].next, props.customAnimation && props.customAnimation.next, { delay: props.animationDelay || 100 }), function (style) { return (createElement("div", { style: __assign({}, styles.item, style), className: "animated-carousel-item" }, props.slides[state.nextActive])); }),
        props.withNavigation ? (createElement("div", { className: "animated-carousel-dots" }, props.slides.map(function (slide, index) { return (createElement("button", { key: "" + index, className: "animated-carousel-dot " + (index === state.nextActive ? "active" : ""), onClick: function (e) { return setIndex(index); } })); }))) : null));
}
var styles = {
    container: {
        position: "relative",
        overflow: "hidden",
        width: "100%",
        height: "100%"
    },
    item: {
        position: "absolute",
        width: "100%",
        height: "100%"
    }
};
var animationTypes = {
    FADE: {
        active: {
            from: { opacity: 1 },
            to: { opacity: 0 }
        },
        next: {
            from: { opacity: 0.19 },
            to: { opacity: 1 }
        }
    },
    SLIDE_LEFT: {
        active: {
            from: { transform: "translateX(0)" },
            to: { transform: "translateX(-100%)" }
        },
        next: {
            from: { transform: "translateX(100%)" },
            to: { transform: "translateX(0%)" }
        }
    },
    SLIDE_RIGHT: {
        active: {
            from: { transform: "translateX(0)" },
            to: { transform: "translateX(100%)" }
        },
        next: {
            from: { transform: "translateX(-100%)" },
            to: { transform: "translateX(0%)" }
        }
    }
};

export default Carousel;
